<?php
/**
*** @name Model_gesa.php
*** @description Model class for gesa
*** @author Quentin George <q.george@esia-sa.com>
*** @version 1.0.0
*** @date 12/06/2023
**/

include_once 'gesa.conf.php';
include_once USR_CLASS;
include_once GESA_CLASS;
include_once LIB_GESA;
class Model_gesa
{
	/**
	 * exécute une requete préparée et traite l'erreur
	 * @param PDOStatement $query un objet PDOStatement (requête préparée)
	 * @param array  $param $param les paramètres à passer à la requête (voir PDOStatement::execute())
	 * @throws EsiaDbException
	 * @retval PDOStatement un objet PDOStatement qui a été exécuté
	 */
	static function executeStatement(PDOStatement $query,$param=NULL)
	{
		$ret = $query->execute($param);
		if($ret === FALSE)
		{
			$err = $query->errorInfo();
			throw new EsiaDbException($err[2]);
		}
		return $query;
	}
	/**
	 * @brief Instancie un utilisateur
	 * @detail  Instancie un utilisateur
	 * @param string $name le login de l'utilisateur
	 * @param string $pwd (optionel) le mot de passe de l'utilisateur (si fourni et que la connection fonctionne set le HashPassword)
	 * @retval User un objet utilisateur
	 * @exception EsiaDbException si une erreur survient
	 */
	static function InstanceUser($name, $pwd='')
	{
		$user = new User(NULL);
		$user->setName($name);

		// Set hash password
		if(strlen($name) > 0 && strlen($pwd) > 0)
		{
			$cmd = 'sudo ' . SHELL_SCRIPTS . 'authenticate.sh ' . escapeshellarg($name) . ' ' . escapeshellarg($pwd);
			exec($cmd, $output, $code);
			if($code == 0)
			{
				$hashPasswd = sha1($name . $pwd);
				$user->setHashPassword($hashPasswd);
			}
		}

		return $user;
	}
	/**
	 * @brief Instancie une GESA
	 * @detail Instancie une GESA
	 * @retval gesa un objet GesaInterface
	 * @exception GesaException si une erreur survient
	 */
	static function instanceGESA()
	{
		$gesa = new GesaInterface();
		// Get Serial Number
		$sn = trim(file_get_contents(SN_FILE));
		$type = '';
		$isVM = FALSE;
		if(!empty($sn))
		{
			$gesa->setSerialNumber($sn);
			// Get gesa type data
			// Sn tag is a 2 digits number which is part of the sn and give us the gesa type (GESA_TYPES_BY_SN_TAG)
			$snTag = substr($sn, 8, 2);
			if(isset(GESA_TYPES_BY_SN_TAG[$snTag]['type']))
			{
				$type = GESA_TYPES_BY_SN_TAG[$snTag]['type'];
			}
			$gesa->setType($type);
			if(isset(GESA_TYPES_BY_SN_TAG[$snTag]['isVM']))
			{
				$isVM = GESA_TYPES_BY_SN_TAG[$snTag]['isVM'];
			}
			$gesa->setIsVM($isVM);
		}

		return $gesa;
	}
	/**
	 * @brief Set GESA common infos parameters
	 * @detail Common params (model, os_version)
	 * @param GesaInterface $gesa un object GesaInterface instancier avec instanceGESA()
	 * @retval gesa un objet GesaInterface
	 * @exception GesaException si une erreur survient
	 */
	static function setGESACommonInfos($gesa)
	{
		// Get model
		if($gesa->getIsVM() == TRUE)
		{
			$gesa->setModel('VM');
		}
		elseif($gesa->getType() == 'Unity')
		{
			// Get rpi model
			$rpi = '';
			$ret = file_get_contents('/proc/device-tree/model');
			if(empty($ret))
			{
				// If no result try the second command
				$ret = file_get_contents('/proc/cpuinfo');
				if(preg_match('/a02082|a22082/', $ret))
				{
					$ret = 'Pi 3 Model B';
				}
				elseif(preg_match('/a020d3/', $ret))
				{
					$ret = 'Pi 3 Model B+';
				}
			}
			else
			{
				// Remove uneccessary chars from the first command result
				$ret = ltrim($ret, 'Raspberry');
			}
			if(!empty($ret))
			{
				$rpi = trim($ret);
			}
			$gesa->setModel($rpi);
		}
		elseif($gesa->getType() == 'Egide Box')
		{
			// Get Egide Box model
		}
		// Get OS version
		$version_os = '';
		$ret = file_get_contents('/etc/debian_version');
		if(!empty($ret) && preg_match('/^[0-9]+.*[0-9]*$/', trim($ret)))
		{
			$version_os = 'Debian ' . trim($ret);
		}
		$gesa->setVersionOS($version_os);

		return $gesa;
	}
	/**
	 * @brief Set GESA common network parameters
	 * @detail Common network params (localIp, netmask, gateway, DNS)
	 * @param GesaInterface $gesa un object GesaInterface instancier avec instanceGESA()
	 * @retval gesa un objet GesaInterface
	 * @exception GesaException si une erreur survient
	 */
	static function setGESANetworkCommon($gesa)
	{
		// Ip addr and netmask from ip addr command because ip route doesn't order interfaces the same way
		$ifaces = getOSValidIfaces();
		if(isset($ifaces[0]))
		{
			$gesa->setLocalIPaddress($ifaces[0]['addr_info'][0]['local']);
			$gesa->setNetmask(netmaskFromCidr($ifaces[0]['addr_info'][0]['prefixlen']));
		}

		// Gateway from ip route command
		$routes = array();
		$ret = shell_exec('ip -j route');
		if(!empty($ret))
		{
			$routes = json_decode($ret, TRUE);
		}
		if(isset($routes[0]))
		{
			$gesa->setGateway($routes[0]['gateway']);
		}

		// DNS from /etc/resolv.conf
		$dnsServers = array();
		$ret = shell_exec('grep -i "^nameserver" /etc/resolv.conf | cut -d " " -f2');
		if(!empty($ret))
		{
			$dnsServers = preg_split("/[\s,]+/", trim($ret));
		}
		$gesa->setDNS($dnsServers);

		return $gesa;
	}
}

class EsiaDbException extends Exception{}
?>
