// common.js

// Self called function to start on file include
(() => {
	document.addEventListener("DOMContentLoaded", () => {
		// Add tooltip styles on every element with title attr. Add tooltip on dialog after openning to avoid bug
		setTooltip($(document), { items: '*[title]:not(.ui-dialog *)' });

		// Handle user messages
		$('#messages').children().each(function()
		{
			fadeUserMessage($(this));
		});
		getSessionMessage();

		setSelectLang();
		setFormClientValidation();
		rebootGesaDialog();
	});
})();

// Set tooltip elements
function setTooltip(element, options={})
{
	element.tooltip({
		classes: { 'ui-tooltip': 'tooltip-wrapper' },
		position: {
			my: 'center bottom-15',
			at: 'center top',
			using: function(position, feedback)
			{
				$(this).css(position);

				// Arrow container should have the same width and position left as the target to always point to it
				const arrow = {
					width: feedback.target.width,
					left: feedback.target.left - $(this).offset().left,
				};

				// Create arrow
				$(this).append($("<div>",
					{
						class: `arrow ${feedback.vertical} ${feedback.horizontal}`,
						style: `width: ${arrow.width}px; left: ${arrow.left}px;`,
					}
				));
			},
		},
		...options,
	});
}

// Annimation to fade an user message
function fadeUserMessage(item)
{
	item.show();
	item.delay(4000).fadeOut(500);
}

// Display error & success messages
function displayUserMessage(errors=[], success=[], idHtml="messages")
{
	const container = $("#" + idHtml);

	// Transform messages to array
	if(!$.isArray(errors))
	{
		errors = [errors];
	}
	if(!$.isArray(success))
	{
		success = [success];
	}

	let messages = [];
	// Create messages
	errors.forEach(err => {
		if(err.length > 0)
		{
			messages.push($('<p class="EventCritical">' + err + '</p>'));
		}
	});
	success.forEach(succ => {
		if(succ.length > 0)
		{
			messages.push($('<p class="EventOK">' + succ + '</p>'));
		}
	});
	// Set messages
	messages.forEach(msg => {
		container.append(msg);
		// Display new message during 5s
		fadeUserMessage(msg);
	});
}

// Add messages to session var to display it at page reload
function setSessionMessage(errors=[], success=[])
{
	// Transform messages to array
	if(!$.isArray(errors))
	{
		errors = [errors];
	}
	if(!$.isArray(success))
	{
		success = [success];
	}

	let messages = sessionStorage.getItem('messages');
	messages = messages ? JSON.parse(messages) : {};

	// Merge data
	messages.errors = messages.errors ? [...messages.errors, ...errors] : errors;
	messages.success = messages.success ? [...messages.success, ...success] : success;

	sessionStorage.setItem("messages", JSON.stringify(messages));
}

// Retrieve saved messages from another page in session var (js session var cannot be accessible by php because it's saved on the browser not on the server)
function getSessionMessage()
{
	let messages = sessionStorage.getItem('messages');

	if(messages) {
		messages = JSON.parse(messages);

		sessionStorage.removeItem('messages');
		displayUserMessage(messages.errors || [], messages.success || []);
	}
}

// Return all inputs, select, ... of an html form as a data object (like used in $.ajax data: {}).
function getFormValues(idHtml) {
	return $('#' + idHtml).serializeArray().reduce(function(o, input) {
		if(input.name.slice(-2) === '[]') {
			// If input name ends by [] its val should be an array
			// Remove [] from the name
			input.name = input.name.slice(0, -2);
			if(!o[input.name]) {
				// If the array key doesn't exist create it and initialize the first array value
				o[input.name] = [input.value];
			} else {
				// The array key already exist push the new value into the array
				o[input.name].push(input.value);
			}
		} else {
			o[input.name] = input.value;
		}

		return o;
	}, {});
}

// Perform a Jquery ajax request with the same behaviour as a common Promise to use it with (async/await statements and error handeling with try catch)
// Usage:
// $.ajax({type: "POST",url: url, ... }).done(data => {return data;});
// ajaxPromise({type: "POST",url: url, ... }).then(data => {return data;});
function ajaxPromise(settings)
{
	return new Promise((resolve, reject) => {
		$.ajax(settings)
		.done(data => resolve(data))
		.fail((jqXHR, textStatus, errorThrown) => reject(errorThrown.message ? errorThrown.message : errorThrown));
	});
}

// Perform an ajax request with response in json usable in try catch async/await statement. Error handeling is preconfigured for php error trow
function ajaxPromiseJSON(settings)
{
	return ajaxPromise({
		type: "POST",
		...settings,
		dataType: "json",
	})
	.then(data => {
		if(data.error) {
			// Error returned by php
			throw data.error;
		} else {
			return data;
		}
	});
}

// Create default custom selects
function setSelect(selector='', options={})
{
	// Init select & return its id
	return new SlimSelect({
		select: selector,
		...options,
		settings: {
			showSearch: false,
			placeholderText: globals.lang.SELECT_VALUE || 'Select value',
			searchText: globals.lang.SELECT_NO_RESULTS || 'No results',
			searchPlaceholder: globals.lang.SELECT_SEARCH || 'Search',
			...options?.settings,
		},
	});
}

// Create language selector
function setSelectLang(selector='.select-lang')
{
	return setSelect(selector, {
		events: {
			afterChange: async ([{ value }]) => {
				try {
					await ajaxPromiseJSON({
						url: 'display.php',
						data: {
							action: 'changeLang',
							lang: value,
						},
					});

					location.reload();
				} catch (err) {
					displayUserMessage(err);
				}
			},
		},
	});
}

// Open dialog window to configure object by ajax method instead of iframe
// url is required but any dialog or ajax configuration parameter can be overhide by passing it into dialogParams or ajaxParams. Example to change dialog width only => dialogParams = {width: 800}
function openDialogByAJAX(url, requestData={}, title='', dialog=$('#dialog-info'), dialogParams={}, ajaxParams={}, isBottomActions=true)
{
	if(typeof dialog == 'string') {
		dialog = $('#' + dialog);
	}

	dialog.addClass('ajaxDialog');
	if(!isBottomActions) {
		dialog.addClass('noBottomActions');
	}

	const setMediaMaxWidth = () => {
		// Set html attribute data-media-maxwidth with a value to be used in css as selector by window size
		// Sizes must be declared by ascending order
		const sizes = dialog.dialog('option').mediaSizes;
		let mediaMaxWidth = 0;

		// Loop on each sizes by ascending order if one is smaller than the width provided set mediaMaxWidth as size compared and break the loop (if width is smaller than 320 it is not neccessary to compare if it smaller than 420).
		for (let i = 0; i < sizes.length; i++) {
			if(dialog.dialog('widget').width() < sizes[i]) {
				mediaMaxWidth = sizes[i];
				break;
			}
		}

		dialog.attr('data-media-maxwidth', mediaMaxWidth);
	}

	const triggerResize = () => {
		dialog.dialog("option").resize();
	}

	dialog.dialog({
		resizable: true,
		modal: true,
		title: title,
		width: 1280,
		height: 'auto',
		maxHeight: 800,
		mediaSizes: [], // Sizes must be declared by ascending order (see setMediaMaxWidth() function)
		...dialogParams,
		open: () => {
			setMediaMaxWidth();

			$(window).on('resize', triggerResize);

			if(typeof dialogParams.open === 'function') {
				dialogParams.open();
			}
		},
		close: () => {
			// On dialog closure remove dialog content to re-create new one on next openning
			dialog.dialog('destroy');
			dialog.children().remove();
			dialog.off('validation');
			dialog.removeClass('ajaxDialog');
			$(window).off('resize', triggerResize);

			if(typeof dialogParams.close === 'function') {
				dialogParams.close();
			}
		},
		resize: () => {
			setMediaMaxWidth();

			if(typeof dialogParams.resize === 'function') {
				dialogParams.resize();
			}
		},
		autoOpen: false,
	});

	// Ajax request to get dialog html content
	$.ajax({
		type: 'POST',
		url: url,
		data: requestData,
		dataType: 'html',
		...ajaxParams
	})
	.done(function(data) {
		if(data)
		{
			dialog.empty(); // Prevent bug on multiple click
			dialog.append(data);
			dialog.find('.btnBack').on('click', function() {
				dialog.dialog('close');
			});
			$('#btnValidDialog').on('click', function() {
				dialog.trigger('validation');
			});

			dialog.dialog('open');
		}
	});
}

function setProgressBar(element, options={})
{
	element.progressbar({
		value: false,
		max: 100,
		...options,
		complete: function() {
			$(this).addClass('completed');

			if(typeof options.complete === 'function')
			{
				options.complete();
			}
		}
	});
}

// Apply css for invalid inputs on form validation
function setFormClientValidation(formSelector='form')
{
	$(formSelector + ' [type="submit"]').on('click focus', function() {
		const form = $(this).closest('form');

		$(':valid', form).removeClass('invalidValue');
		$(':invalid', form).addClass('invalidValue');
	});
}

// Copy to clipboard function with compatibility for old browser and http sites. Use it inside try/catch in async await function
async function copyToClipboardCompat(text)
{
	try {
		if(navigator?.clipboard?.writeText != undefined)
		{
			// If present use new navigator.clipboard API
			await navigator.clipboard.writeText(text);
		}
		else if(document.queryCommandSupported && document.queryCommandSupported("copy"))
		{
			// If no navigator.clipboard API use old deprecated document.execCommand("copy")
			let textarea = document.createElement("textarea");
			
			textarea.textContent = text;
			textarea.style.position = "fixed"; // Prevent scrolling to bottom of page
			document.body.appendChild(textarea);
			textarea.select();
			try {
				// Security exception may be thrown by some browsers
				document.execCommand("copy");
			}
			finally {
				document.body.removeChild(textarea);
			}
		}
		else
		{
			throw 'No copy API found !';
		}
	} catch (err) {
		throw err.message ? err.message : err;
	}
}

// Copy an jquery element content on click. With styles and error handeling
function copyEltTextOnClick(eltClickable, eltChildSelector, errMsg, textBefore='', textAfter='')
{
	eltClickable.addClass('contentCopy');

	eltClickable.on('click', async function() {
		const text = textBefore + $(eltChildSelector, $(this)).html() + textAfter;

		// Visual animation on cursor during 300ms
		$(this).css('cursor', 'progress');
		setTimeout(() => {
			$(this).css('cursor', '');
		}, '300');
		  
		try {
			await copyToClipboardCompat(text);
		} catch (err) {
			let msg = errMsg ? errMsg : 'Error copy'; // If not given or empty set default msg
			
			displayUserMessage(msg + ' (' + text + '): ' + err);
		}
	});
}

// Reload the current tab of jquery-ui tabs
function reloadActiveTab(tabsElt)
{
	const activeIndex = tabsElt.tabs('option', 'active');
	tabsElt.tabs('load', activeIndex);
}

// Retreive the saved active tab of jquery-ui tabs
function getSessionActiveTab(idHtml)
{
	let activeIndex = sessionStorage.getItem('activeTab-' + idHtml);
	if(isNaN(activeIndex) || activeIndex < 0)
	{
		activeIndex = 0;
	}

	return activeIndex;
}

// Creat a jquery-ui tabs with persistant active tab on any page reload
function createTabsPersistant(idHtml, options={})
{
	$('#' + idHtml).tabs({
		...options,
		active: getSessionActiveTab(idHtml),
		activate: function(e, ui) {
			sessionStorage.setItem('activeTab-' + idHtml, $(this).tabs('option', 'active'));

			if(typeof options.activate === 'function')
			{
				options.activate(e, ui);
			}
		},
	});
}

// Calc time duration in seconds from startTime in ms to now and return seconds from class instance or restart method to now with getTimeDuration method
function TimeDuration()
{
	this.startTime = performance.now();
	this.restart = () => {
		this.startTime = performance.now();
	};
	this.getTimeDuration = () => {
		return Math.floor((performance.now() - this.startTime) / 1000);
	};
}

/** @description: Check if the server is availlable (use if GESA reboot). Will perform an ajax basic request with timeout *timeout* every *retryIntervalSec* with a *maxRetries* number (timeout == 3, retryIntervalSec == 5, maxRetries = 0 => try connection to server during 3 seconds every 5 seconds during 5 minutes). Resolve when server respond and return duration in seconds. Reject if timeout exceeded and throw duration in seconds.
*** @param object requestParams={timeout: 3, retryIntervalSec: 5, maxRetries: 60, execNow: false}
*** @param object timeDuration=TimeDuration() class object
*** @param object ajaxSettings=jquery ajax function settings
*** @return string Success message if ajax request return successfully (server is availlable)
*** @throws Exception if timeout exceed
**/
async function checkServerAvaillability(requestParams={}, timeDuration={}, ajaxSettings={})
{
	return new Promise((resolve, reject) => {
		// Check function arguments & limit values
		if(!requestParams?.timeout || requestParams.timeout < 3)
		{
			requestParams.timeout = 3;
		}
		// Min retry value to let the ajax request time to execute with timeout and gap of 2 seconds
		const minRetryInterval = requestParams.timeout + 2;
		if(!requestParams?.retryIntervalSec || requestParams.retryIntervalSec < minRetryInterval)
		{
			requestParams.retryIntervalSec = minRetryInterval;
		}
		if(!requestParams?.maxRetries)
		{
			requestParams.maxRetries = 0;
		}
		// Instance TimeDuration object if not passed in arguments
		if(!timeDuration?.startTime)
		{
			timeDuration = new TimeDuration();
		}

		// Logic to execute at every interval
		let retriesCounter = 0;
		const task = async () => {
			try {
				await new Promise((res, rej) => {
					$.ajax({
						url: 'index.php',
						processData: false,
						...ajaxSettings,
						timeout: requestParams.timeout * 1000, // seconds to milliseconds
					})
					.done(() => res()) // Url availlable
					.fail((jqXHR, textStatus, errorThrown) => {
						if(textStatus == 'timeout')
						{
							rej(errorThrown.message ? errorThrown.message : errorThrown); // Go to catch et check for retries
						}
						else
						{
							res(); // Url availlable
						}
					});
				});

				clearInterval(interval);
				resolve(globals.lang.TASK_COMPLETED_AFTER + ' ' + timeDuration.getTimeDuration() + ' ' + globals.lang.SECONDS + ' !');
			} catch (err) {
				if(retriesCounter >= requestParams.maxRetries)
				{
					clearInterval(interval);
					reject(globals.lang.MAX_TIMEOUT_EXCEEDED_AFTER + ' ' + timeDuration.getTimeDuration() + ' ' + globals.lang.SECONDS + ' ! ' + err);
				}
				retriesCounter++;
			}
		};

		// Execute the task by interval
		let interval = setInterval(async () => task(), requestParams.retryIntervalSec * 1000);

		// If the task must execute at start or else after first timer by interval function
		if(requestParams?.execNow == true)
		{
			task();
		}
	});
}

// Handle reboot gesa button
function rebootGesaDialog()
{
	$('#rebootBtn').on('click', async () => {
		openDialogByAJAX(
			'dialog.php',
			{ 
				action: 'reboot',
			},
			globals.lang['REBOOT_GESA'],
			$('#dialog-info'),
			{
				resizable: false,
				draggable: true,
				width: 800,
			},
			{},
			true,
		);
	});
}

// Execute server command and return output in live using server-sent events to use with javascript EventSource for client display
function execWithLiveReturn(url, htmlElt=null, fLineCallback=null)
{
	return new Promise((resolve, reject) => {
		if(typeof(EventSource) === 'undefined')
		{
			reject('Your browser does not support server-sent events. It cannot display live console.');
		}
		else
		{
			var evtSource = new EventSource(url);
			evtSource.onmessage = ({ data }) => {
				if(data == '#!*@&ENDENDEND&@*!#')
				{
					// End line founded. The cmd return is finished
					evtSource.close();
					resolve();
				}
				else
				{
					if(htmlElt)
					{
						htmlElt.innerHTML += data + '<br>';
						htmlElt.scrollTo(htmlElt.scrollLeft, htmlElt.scrollHeight);
					}
					if(typeof fLineCallback === 'function')
					{
						fLineCallback(data);
					}
				}
			};
			evtSource.onerror = () => {
				if(evtSource.readyState === EventSource.CLOSED)
				{
					evtSource.close();
					reject('An error occurred preventing console output from being displayed.');
				}
			};
		}
	});
}
